/**
@file
@verbatim
$Id: FormatInfo.h 3096 2005-06-21 20:07:11Z ekuznetsov $

Copyright (c) 2002-2004 DivX, Inc. All rights reserved.

This software is the confidential and proprietary information of DivX, Inc.,
Inc. and may be used only in accordance with the terms of your license from
DivX, Inc.

@endverbatim
*/

#ifndef _Q_FormatInfo_h
#define _Q_FormatInfo_h

#include "./FourCC.h"

#ifdef __cplusplus
extern "C" {
#endif

/// Describes a compressed or uncompressed video format.
typedef struct FormatInfo
{
    /// Four CC of the video format.
    FourCC fourCC;

    /// Bits per pixel (zero if not known).  This field is used to distinguish the 
    /// various uncompressed RGB formats.
    int bpp;

    /// Width of the image in pixels.
    int width;
    
    /// Height of the image in pixels.
    int height; 

    /// Set non-zero if the bottom line of the image appears first in the buffer.
    int inverted;

    /// Pixel aspect ratio:  horizontal part.
    int pixelAspectX;

    /// Pixel aspect ratio:  vertical part.
    int pixelAspectY;

    /// Maximum size in bytes of a video frame of this format.
    int sizeMax;

    /// Number of units of time in a second.
    int timescale;

    /// Duration of each frame, in units of time defined by timescale.  In the case 
    /// of variable framerate, this should be set to the maximum expected frame period.
    int framePeriod;

    /// 1 if frame rate is constant; 0 otherwise.
    int framePeriodIsConstant;
} 
FormatInfo;

/// Convenience function.
/// @param pFormatInfo Pointer to a populated FormatInfo structure.  
/// @return The total number of luminance pictures in each video frame.
static int FormatInfo_getTotalPixels(const FormatInfo* pFormatInfo) 
{ 
    return pFormatInfo->height * pFormatInfo->width; 
}

/// Convenience function.  Only works when bpp is known and set correctly.
/// @param pFormatInfo Pointer to a populated FormatInfo structure.
/// @return The size in bytes of a single video frame.
static int FormatInfo_getFrameSize(const FormatInfo* pFormatInfo) 
{ 
    return FormatInfo_getTotalPixels(pFormatInfo) * pFormatInfo->bpp / 8;; 
}

/// Convenience function.
/// @param pFormatInfo Pointer to a populated FormatInfo structure.
/// @return The framerate of the format according to framePeriod and timescale members.
static double FormatInfo_getFramerate(const FormatInfo* pFormatInfo) 
{ 
    return (double)pFormatInfo->timescale / (double)pFormatInfo->framePeriod; 
}

static int FormatInfo_diff(const FormatInfo* pFormat1, const FormatInfo* pFormat2)
{
    return
        (pFormat1->fourCC != pFormat2->fourCC) ||
        (pFormat1->fourCC == 0 && pFormat1->bpp != pFormat2->bpp) ||
        (pFormat1->width != pFormat2->width) ||
        (pFormat1->height != pFormat2->height) ||
        (pFormat1->inverted != pFormat2->inverted) ||
        (pFormat1->pixelAspectX != pFormat2->pixelAspectX) ||
        (pFormat1->pixelAspectY != pFormat2->pixelAspectY) ||
        (pFormat1->timescale != pFormat2->timescale) ||
        (pFormat1->framePeriodIsConstant != pFormat2->framePeriodIsConstant) ||
        (pFormat1->framePeriodIsConstant && (pFormat1->framePeriod != pFormat2->framePeriod));
}

#ifdef __cplusplus
}
#endif

#endif // _Q_FormatInfo_h

