/**
@file
@verbatim
$Id: EncoderInterface.h 4802 2005-11-01 01:26:28Z jfunnell $

Copyright (c) 2003-2005 DivX, Inc. All rights reserved.

This software is the confidential and proprietary information of DivX, Inc.,
Inc. and may be used only in accordance with the terms of your license from
DivX, Inc.

@endverbatim
*/

#ifndef _Q_LibQEnc_h
#define _Q_LibQEnc_h

#include "./FrameInput.h"
#include "./FrameOutput.h"
#include "./FrameResult.h"
#include "./Settings.h"
#include "./FeedbackInterface.h"
#include "./EncoderCallback.h"

#include "../common/FormatInfo.h"


/// The external encoder API.
/// @todo Example usage  
class EncoderInterface
{
public:
    /// Method for creation of an encoder.  IQEncoderInterface::destroy() must
    /// later be called in order to free memory.
    /// @return Pointer to newly created encoder instance or null on failure.
    static EncoderInterface* create();

    /// Method for destruction of an encoder.
    /// @param pIQEncoderInterface Pointer to encoder instance to be destoyed.
    static void destroy(EncoderInterface* pIQEncoderInterface);

protected:
    virtual ~EncoderInterface() {}

public:

    /// Used to access the encoder's settings API.
    /// @return Pointer to Settings interface.
    virtual Settings* getSettingsApi() = 0;

    /// This function provides the encoder with a pointer to the object
    /// that will receive encoding feedback information.
    /// @param pFeedbackInterface Feedback object pointer.
    virtual void setFeedback(FeedbackInterface* pFeedbackInterface) = 0;

    virtual void setEncoderCallback(EncoderCallback* pCallback) = 0;

    /// Returns version information about this build of the codec.
    /// @return Textual build information, for example "06_00_00_00000_DIVX_CODEC".
    virtual const char* getVersionInfo() const = 0;

    /// Tells encoder about colorspace and dimensions of video image.
    /// Cannot be called after delivering the first video frame for encoding.
    /// @param pFormatInfo Pointer to a structure descrbing the input format.
    virtual void setFormatInput(const FormatInfo* pFormatInfo) = 0;

    /// Gets information about the input format.  This should be exactly the same
    /// as was previously set using setFormatInput().
    /// @param pFormatInfo Pointer to a structure that will receive the input format.
    /// @return True on success.  False indicates a bad input format.
    virtual bool getFormatInput(FormatInfo* pFormatInfo) const = 0;

    /// Gets information about output format
    /// @param pFormatInfo Pointer to a structure that will receive the output format.
    /// @return True on success.  False indicates a bad input format and/or
    /// bad settings (e.g. illegal resize dimensions).
    virtual bool getFormatOutput(FormatInfo* pFormatInfo) const = 0;

public: // Delivery API 

    /// Delivers an uncompressed frame to the encoder.  Input frame
    /// must be maintained valid until either encodeFrame() or
    /// encode() has been called.
    /// Call with pFrameInput->m_pFrameT=0 to indicate end-of-stream.
    /// @param pFrameInput The input frame.
    virtual void deliverFrame(const FrameInput* pFrameInput) = 0;

public: // Bitstream API #1

    /// Produces a single compressed frame in bitstream order.  Caller must handle bitstream processing and packing.
    /// @param pFrameOutput Pointer to structure where the encoder should write the compressed output frame.
    /// @param pFrameResult Optional pointer to results structure that will be populated by the encoder.
    /// @return True if the encoder encoded a frame.  A false return value most likely means that the 
    /// encoder is waiting for an input frame to be delivered.
    virtual bool encodeFrame(FrameOutput* pFrameOutput, FrameResult* pFrameResult=0) = 0;

public: // Bitstream API #2

    /// Encodes a single chunk of packed bitstream, containing zero, one or two compressed frames.
    /// This function must be called repeatedly - on each call it encodes one frame.  Only when 
    /// it returns true is the chunk ready for output.  
    /// @param pFrameOutput Pointer to structure where the encoder should write the compressed output chunk.
    /// @param pFrameResult Optional pointer a results structure that will be populated by the encoder.
    /// @return True if the encoder has finished encoding the output chunk.  False if it has more work to do.
    virtual bool encode(FrameOutput* pFrameOutput, FrameResult* pFrameResult=0) = 0;
};


#endif
