/**
@file
@verbatim
$Id: Settings.h 4691 2005-10-21 15:31:49Z jfunnell $

Copyright (c) 2003-2005 DivX, Inc. All rights reserved.

This software is the confidential and proprietary information of DivX, Inc.,
Inc. and may be used only in accordance with the terms of your license from
DivX, Inc.

@endverbatim
*/

#ifndef _Q_Settings_h
#define _Q_Settings_h

#include "../common/DivXPortable.h"
#include "../common/FourCC.h"

#include <string.h>



#define SETTINGS_REVISION "$Revision: 4691 $"


/**
@brief API for controlling codec settings.  

This abstract class provides access to a set of parameters.  Each parameter has a
type, a name and a value.  This API is designed to manage interactions between 
parameters and to enforce restrictions on allowed values.

Each parameter has a predefined type so the codec can expose integer, double, 
boolean, string and enumerated parameters.  Parameters may be marked as disabled 
or read-only by the codec, and they each have a default value.  Numeric paremters
have minimum and maximum limits.  Enumerated parameters have a "mask" of allowable
values.

Parameters are implicitly ordered by precedence.  This allows the user to make
certain assumtions about the behavior of the Settings API.  Modifiying a
paramater's value could affect parameters of lower precedence - their values could 
change, they could be enabled or disabled or their range or mask might be
changed.  Parameters of higher precedence that the modified parameter are 
guaranteed not to be affected.

The settings are guaranteed to be in a usable state after every user interaction.
This means that some interactions will fail, for example attempting to change
a disabled or read-only parameter.

@todo Documentation: example usage
*/
class Settings
{
protected:
    Settings(const Settings& rhs);
    Settings();
    virtual ~Settings();
public:

    /// Allows user to copy settings from one codec object to another.
    Settings& operator=(const Settings& rhs);

public:

    /// Parameter type.  Each parameter takes of one of these types.
    typedef enum
    {
        BOOLEAN, ///< Boolean parameter.
        INTEGER, ///< Integer parameter.
        DOUBLE,  ///< Double parameter.
        ENUM,    ///< Enumerated parameter.
        STRING,  ///< ASCII string parameter.
        DATA     ///< Binary data parameter.
    } 
    Type;

    /// Parameter name.  Each parameter has a unique name.  
    class Name
    {
    public:
        Name(const char* str=0)
        {
            this->str = str;
        }
        bool operator==(const Name& rhs) const
        {
            return !strcmp(this->str, rhs.str);
        }
        bool operator!=(const Name& rhs) const
        {
            return !!strcmp(this->str, rhs.str);
        }
        bool operator<(const Name& rhs) const
        {
            return strcmp(this->str, rhs.str) > 0;
        }
        const char* getStr() const
        {
            return this->str;
        }
        bool isValid() const
        {
            return !!this->str;
        }
    private:
        const char* str;
    };

    /// Type used for the value of enumerated (ENUM) parameters.
    typedef uint32_t Enum;

    /// C++ exception type used by the settings API.
    class Exception
    {
    public:
        Exception(const char* info)
        {
            this->info = info;
        }
        const char* getErrorMessage() const
        {
            return this->info;
        }
    private:
        const char* info;
    };

public:

    /// Used to get a list of the names of the currently active parameters.
    /// @return Name of parameter in precedence position n. If n is out of range,
    /// an invalid name will be returned.
    virtual Name getName(int n) const = 0;

    /// Queries the type of the specified paramter.
    /// @param name Name of the parameter in question.
    /// @return Type of the parameter.
    virtual Type getType(Name name) const = 0;

    /// Queries whether the specified paramter is read-only.
    /// @param name Name of the parameter in question.
    /// @return True if the parameter is read-only.
    virtual bool isReadOnly(Name name) const = 0;

    /// Queries whether the specified paramter is enabled.
    /// @param name Name of the parameter in question.
    /// @return True if the parameter is enabled.
    virtual bool isEnabled(Name name) const = 0;

    /// Queries whether the specified paramter is equal to its default value.
    /// @param name Name of the parameter in question.
    /// @return True if the parameter's value is the same as its default value.
    virtual bool isDefault(Name name) const = 0;

    /// Sets the value of a BOOLEAN parameter.
    /// @param name Name of the parameter in question.
    /// @param value The parameter's intended value.
    virtual void setBool(Name name, bool value) = 0;

    /// Queries the value of a BOOLEAN parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current value.
    virtual bool getBool(Name name) const = 0;

    /// Sets the value of an INTEGER parameter.
    /// @param name Name of the parameter in question.
    /// @param value The parameter's intended value.
    virtual void setInt(Name name, int value) = 0;

    /// Queries the value of an INTEGER parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current value.
    virtual int getInt(Name name) const = 0;

    /// Queries the minimum value of an INTEGER parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current minimum allowable value.
    virtual int getIntMin(Name name) const = 0;

    /// Queries the maximum value of an INTEGER parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current maximum allowable value.
    virtual int getIntMax(Name name) const = 0;

    /// Sets the value of a DOUBLE parameter.
    /// @param name Name of the parameter in question.
    /// @param value The parameter's intended value.
    virtual void setDouble(Name name, double value) = 0;

    /// Queries the value of a DOUBLE parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current value.
    virtual double getDouble(Name name) const = 0;

    /// Queries the minimum value of a DOUBLE parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current minimum allowable value.
    virtual double getDoubleMin(Name name) const = 0;

    /// Queries the maximum value of a DOUBLE parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current maximum allowable value.
    virtual double getDoubleMax(Name name) const = 0;

    /// Sets the value of an ENUM parameter.
    /// @param name Name of the parameter in question.
    /// @param value The parameter's intended value.
    virtual void setEnum(Name name, Enum value) = 0;

    /// Queries the value of an ENUM parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current value.
    virtual Enum getEnum(Name name) const = 0;

    /// Queries the current allowable settings for an ENUM parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current mask value.
    virtual Enum getEnumMask(Name name) const = 0;

    /// Sets the value of an STRING parameter.
    /// The codec will make its own copy of the string value.
    /// @param name Name of the parameter in question.
    /// @param value The parameter's intended value, as a null-terminated string.
    virtual void setStr(Name name, const char* value) = 0;

    /// Queries the value of a STRING parameter.
    /// @param name Name of the parameter in question.
    /// @return The parameter's current value, as a null-terminated string.
    virtual const char* getStr(Name name) const = 0;

    /// Sets the value of a DATA parameter.
    /// The codec will make its own copy of the data value.
    /// @param name Name of the parameter in question.
    /// @param buff Pointer to a buffer containing the intended data value.
    /// @param len Length of the data in bytes.
    virtual void setData(Name name, const uint8_t* buff, int len) = 0;

    /// Queries the value of a DATA parameter.
    /// @param name Name of the parameter in question.
    /// @return Pointer to a buffer containing the current data value.
    virtual const uint8_t* getData(Name name) const = 0;

    /// Queries the length of a DATA parameter.
    /// @param name Name of the parameter in question.
    /// @return Length of the current data value in bytes.
    virtual int getDataLen(Name name) const = 0;

    /// Uses the current set of parameter values as the new default settings.
    /// Current parameter values are unaffected but the previous set of default
    /// values will be lost.  The next call to resetDefaults() will reset all 
    /// parameters to the values at the time this method was called.
    virtual void makeCurrentSettingsDefault() = 0;

    /// Resets all parameters to their default values. 
    virtual void resetDefaults() = 0;

    /// Detirmines a parameter's position in the precedence hierarchy.
    /// @param name Name of the parameter in question.
    /// @return Precedence of the specified parameter, higher value signifies
    /// higher precedence.
    virtual int getPrecedence(Settings::Name name) const = 0;

public:

    /// Convenience function:  converts a FourCC compression type to the
    /// appropriate ENUM parameter value.
    /// @param fourCC FourCC video format type.
    /// @return ENUM value suitable for use with Settings::setEnum().
    static Enum fourCC2Encoder(FourCC fourCC);

    /// Checks whether the user has the correct version of Settings.h.
    /// Call this method with no explicit parameters, allowing the default
    /// to be picked up.  Any user of this header file may use this
    /// function to test, for example, that the macro definitions
    /// are current.
    /// @param version Must be left undefined so that it takes its default
    /// value from Settings.h.
    /// @return True if the correct version of Settings.h is being used.
    static bool isCorrectHeader(const char* version=SETTINGS_REVISION);
};



/* Encoder modes for "encoder" */
#define ENCODER_IYUV  (Settings::Enum(1<<2))
#define ENCODER_MPEG4 (Settings::Enum(1<<4))

/* Encoder modes for "profile" */
#define PROFILE_UNCONSTRAINED (Settings::Enum(1<<0))
#define PROFILE_HANDHELD      (Settings::Enum(1<<1))
#define PROFILE_PORTABLE      (Settings::Enum(1<<2))
#define PROFILE_HOME_THEATER  (Settings::Enum(1<<3))
#define PROFILE_HDTV          (Settings::Enum(1<<4))

/* Performance / quality mode for "performance" */
#define PERFORMANCE_FASTEST   6950
#define PERFORMANCE_FASTER    7110
#define PERFORMANCE_FAST      7270
#define PERFORMANCE_STANDARD  7670
#define PERFORMANCE_SLOW      8100
#define PERFORMANCE_VERYSLOW  8610

/* Rate control modes for "rcmode" */
#define RCMODE_1PASS_CONSTANT_Q     (Settings::Enum(1<<0)) /**< rcmode value for 1-pass constant quality */
#define RCMODE_VBV_1PASS            (Settings::Enum(1<<1)) /**< rcmode value for VBV 1-pass */
#define RCMODE_VBV_MULTIPASS_1ST    (Settings::Enum(1<<2)) /**< rcmode value for VBV multipass 1st-pass */
#define RCMODE_VBV_MULTIPASS_NTH    (Settings::Enum(1<<3)) /**< rcmode value for VBV multipass nth-pass */
#define RCMODE_VBV_1PASS_CQ         (Settings::Enum(1<<4)) /**< rcmode value for VBV 1-pass constant quality (experimental) */

/* Resize modes for "resize_mode" */
#define RESIZE_MODE_OFF         (Settings::Enum(1<<0))
#define RESIZE_MODE_AUTOHEIGHT  (Settings::Enum(1<<1))
#define RESIZE_MODE_ON          (Settings::Enum(1<<2))

/* Resize modes for "resize_algorithm" */
#define RESIZE_ALGORITHM_BILINEAR        (Settings::Enum(1<<0))
#define RESIZE_ALGORITHM_BICUBIC_SOFT    (Settings::Enum(1<<1))
#define RESIZE_ALGORITHM_BICUBIC_NORMAL  (Settings::Enum(1<<2))
#define RESIZE_ALGORITHM_BICUBIC_SHARP   (Settings::Enum(1<<3))
#define RESIZE_ALGORITHM_LANCZOS4        (Settings::Enum(1<<5))

/* Pixel aspect ratio modes */
#define PAR_1_1           (Settings::Enum(1<<1))
#define PAR_12_11         (Settings::Enum(1<<2))
#define PAR_10_11         (Settings::Enum(1<<3))
#define PAR_16_11         (Settings::Enum(1<<4))
#define PAR_40_33         (Settings::Enum(1<<5))
#define PAR_AUTO          (Settings::Enum(1<<6))
#define PAR_AUTO_NOT_EXT  (Settings::Enum(1<<7))
#define PAR_UNKNOWN       (Settings::Enum(1<<8))

/* Noise reduction filter modes for "filter_mode" */
#define FILTER_MODE_OFF     (Settings::Enum(1<<1))
#define FILTER_MODE_LIGHT   (Settings::Enum(1<<2))
#define FILTER_MODE_NORMAL  (Settings::Enum(1<<3))
#define FILTER_MODE_STRONG  (Settings::Enum(1<<4))
#define FILTER_MODE_EXTREME (Settings::Enum(1<<5))
#define FILTER_MODE_CUSTOM  (Settings::Enum(1<<6))

/* Quantizer modes for "quantization" */
#define QUANTIZATION_H263       (Settings::Enum(1<<1))
#define QUANTIZATION_H263_OPT   (Settings::Enum(1<<2))
#define QUANTIZATION_MPEG       (Settings::Enum(1<<3))

/* Psychovisual modes for "psychovisual" */
#define PSYCHOVISUAL_OFF       (Settings::Enum(1<<1))
#define PSYCHOVISUAL_SHAPING   (Settings::Enum(1<<2))
#define PSYCHOVISUAL_MASKING   (Settings::Enum(1<<3))

/* PSNR computation modes for "psnr_mode" */
#define PSNR_MODE_LUMA         (Settings::Enum(1<<1))
#define PSNR_MODE_LUMA_CHROMA  (Settings::Enum(1<<2))
#define PSNR_MODE_MODULATED    (Settings::Enum(1<<3))
#define PSNR_MODE_OFF          (Settings::Enum(1<<4))

#endif
